/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "ImageData.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomImageData.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomImageData.h"
#include "gdom/PrivateIntGdomImageData.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::ImageData* obj)
{
    return WebCore::cacheDOMObject<WebCore::ImageData, GDOMImageData>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMImageData::GDOMImageData(PassRefPtr<WebCore::ImageData> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMImageData::~GDOMImageData()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "ImageData.h"
#include <gdom/GdomImageData.h>
#include <gdom/PrivateGdomImageData.h>
#include <gdom/PrivateIntGdomImageData.h>
#include <gdom/GdomImageDataprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomImageData, gdom_image_data, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomImageDataPrivate {
    class WebCore::ImageData *item;
};

#define GDOM_IMAGE_DATA_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_IMAGE_DATA, GdomImageDataPrivate))
*/
G_END_DECLS
WebCore::ImageData *
gdom_image_data_get_private(GdomImageData *request)
{
    GDOMImageData *res;
    res = static_cast<GDOMImageData*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::ImageData*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_WIDTH,
    PROP_HEIGHT,
    PROP_DATA,
};


static void gdom_image_data_finalize(GObject* object)
{
    /*GdomImageData* request = GDOM_IMAGE_DATA(object);*/
    /*GdomImageDataPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_image_data_parent_class)->finalize(object);
}

static void gdom_image_data__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomImageData *self = GDOM_IMAGE_DATA(object);
    WebCore::ImageData * gself = (gdom_image_data_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_image_data__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomImageData *self = GDOM_IMAGE_DATA(object);
    WebCore::ImageData * gself = (gdom_image_data_get_private(self));
    switch (prop_id) {
    case PROP_WIDTH:
    {
         g_value_set_long (value, gself->width());
         break;
    }
    case PROP_HEIGHT:
    {
         g_value_set_long (value, gself->height());
         break;
    }
    case PROP_DATA:
    {
         PassRefPtr<WebCore::CanvasPixelArray> ptr = gself->data();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_image_data_class_init(GdomImageDataClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_image_data_finalize;
    g_klass->set_property = gdom_image_data__set_property_fn;
    g_klass->get_property = gdom_image_data__get_property_fn;

    g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_long ("width", /* name */
                "image_data_width", /* short description */
                "read-only  glong ImageData.width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_HEIGHT,
            g_param_spec_long ("height", /* name */
                "image_data_height", /* short description */
                "read-only  glong ImageData.height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DATA,
            g_param_spec_object ("data", /* name */
                "image_data_data", /* short description */
                "read-only  GdomCanvasPixelArray * ImageData.data", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CANVAS_PIXEL_ARRAY, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomImageDataPrivate));*/
}

static void gdom_image_data_init(GdomImageData* request)
{
    /*GdomImageDataPrivate* priv = GDOM_IMAGE_DATA_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

